<?php
/**
 * Theme functions and definitions
 *
 * @package HelloElementor
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

define( 'HELLO_ELEMENTOR_VERSION', '2.6.1' );

if ( ! isset( $content_width ) ) {
	$content_width = 800; // Pixels.
}

if ( ! function_exists( 'hello_elementor_setup' ) ) {
	/**
	 * Set up theme support.
	 *
	 * @return void
	 */
	function hello_elementor_setup() {
		if ( is_admin() ) {
			hello_maybe_update_theme_version_in_db();
		}

		$hook_result = apply_filters_deprecated( 'elementor_hello_theme_load_textdomain', [ true ], '2.0', 'hello_elementor_load_textdomain' );
		if ( apply_filters( 'hello_elementor_load_textdomain', $hook_result ) ) {
			load_theme_textdomain( 'hello-elementor', get_template_directory() . '/languages' );
		}

		$hook_result = apply_filters_deprecated( 'elementor_hello_theme_register_menus', [ true ], '2.0', 'hello_elementor_register_menus' );
		if ( apply_filters( 'hello_elementor_register_menus', $hook_result ) ) {
			register_nav_menus( [ 'menu-1' => __( 'Header', 'hello-elementor' ) ] );
			register_nav_menus( [ 'menu-2' => __( 'Footer', 'hello-elementor' ) ] );
		}

		$hook_result = apply_filters_deprecated( 'elementor_hello_theme_add_theme_support', [ true ], '2.0', 'hello_elementor_add_theme_support' );
		if ( apply_filters( 'hello_elementor_add_theme_support', $hook_result ) ) {
			add_theme_support( 'post-thumbnails' );
			add_theme_support( 'automatic-feed-links' );
			add_theme_support( 'title-tag' );
			add_theme_support(
				'html5',
				[
					'search-form',
					'comment-form',
					'comment-list',
					'gallery',
					'caption',
					'script',
					'style',
				]
			);
			add_theme_support(
				'custom-logo',
				[
					'height'      => 100,
					'width'       => 350,
					'flex-height' => true,
					'flex-width'  => true,
				]
			);

			/*
			 * Editor Style.
			 */
			add_editor_style( 'classic-editor.css' );

			/*
			 * Gutenberg wide images.
			 */
			add_theme_support( 'align-wide' );

			/*
			 * WooCommerce.
			 */
			$hook_result = apply_filters_deprecated( 'elementor_hello_theme_add_woocommerce_support', [ true ], '2.0', 'hello_elementor_add_woocommerce_support' );
			if ( apply_filters( 'hello_elementor_add_woocommerce_support', $hook_result ) ) {
				// WooCommerce in general.
				add_theme_support( 'woocommerce' );
				// Enabling WooCommerce product gallery features (are off by default since WC 3.0.0).
				// zoom.
				add_theme_support( 'wc-product-gallery-zoom' );
				// lightbox.
				add_theme_support( 'wc-product-gallery-lightbox' );
				// swipe.
				add_theme_support( 'wc-product-gallery-slider' );
			}
		}
	}
}
add_action( 'after_setup_theme', 'hello_elementor_setup' );

function hello_maybe_update_theme_version_in_db() {
	$theme_version_option_name = 'hello_theme_version';
	// The theme version saved in the database.
	$hello_theme_db_version = get_option( $theme_version_option_name );

	// If the 'hello_theme_version' option does not exist in the DB, or the version needs to be updated, do the update.
	if ( ! $hello_theme_db_version || version_compare( $hello_theme_db_version, HELLO_ELEMENTOR_VERSION, '<' ) ) {
		update_option( $theme_version_option_name, HELLO_ELEMENTOR_VERSION );
	}
}

if ( ! function_exists( 'hello_elementor_scripts_styles' ) ) {
	/**
	 * Theme Scripts & Styles.
	 *
	 * @return void
	 */
	function hello_elementor_scripts_styles() {
		$enqueue_basic_style = apply_filters_deprecated( 'elementor_hello_theme_enqueue_style', [ true ], '2.0', 'hello_elementor_enqueue_style' );
		$min_suffix          = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		if ( apply_filters( 'hello_elementor_enqueue_style', $enqueue_basic_style ) ) {
			wp_enqueue_style(
				'hello-elementor',
				get_template_directory_uri() . '/style' . $min_suffix . '.css',
				[],
				HELLO_ELEMENTOR_VERSION
			);
		}

		if ( apply_filters( 'hello_elementor_enqueue_theme_style', true ) ) {
			wp_enqueue_style(
				'hello-elementor-theme-style',
				get_template_directory_uri() . '/theme' . $min_suffix . '.css',
				[],
				HELLO_ELEMENTOR_VERSION
			);
		}
	}
}
add_action( 'wp_enqueue_scripts', 'hello_elementor_scripts_styles' );

if ( ! function_exists( 'hello_elementor_register_elementor_locations' ) ) {
	/**
	 * Register Elementor Locations.
	 *
	 * @param ElementorPro\Modules\ThemeBuilder\Classes\Locations_Manager $elementor_theme_manager theme manager.
	 *
	 * @return void
	 */
	function hello_elementor_register_elementor_locations( $elementor_theme_manager ) {
		$hook_result = apply_filters_deprecated( 'elementor_hello_theme_register_elementor_locations', [ true ], '2.0', 'hello_elementor_register_elementor_locations' );
		if ( apply_filters( 'hello_elementor_register_elementor_locations', $hook_result ) ) {
			$elementor_theme_manager->register_all_core_location();
		}
	}
}
add_action( 'elementor/theme/register_locations', 'hello_elementor_register_elementor_locations' );

if ( ! function_exists( 'hello_elementor_content_width' ) ) {
	/**
	 * Set default content width.
	 *
	 * @return void
	 */
	function hello_elementor_content_width() {
		$GLOBALS['content_width'] = apply_filters( 'hello_elementor_content_width', 800 );
	}
}
add_action( 'after_setup_theme', 'hello_elementor_content_width', 0 );

if ( is_admin() ) {
	require get_template_directory() . '/includes/admin-functions.php';
}

/**
 * If Elementor is installed and active, we can load the Elementor-specific Settings & Features
*/

// Allow active/inactive via the Experiments
require get_template_directory() . '/includes/elementor-functions.php';

/**
 * Include customizer registration functions
*/
function hello_register_customizer_functions() {
	if ( is_customize_preview() ) {
		require get_template_directory() . '/includes/customizer-functions.php';
	}
}
add_action( 'init', 'hello_register_customizer_functions' );

if ( ! function_exists( 'hello_elementor_check_hide_title' ) ) {
	/**
	 * Check hide title.
	 *
	 * @param bool $val default value.
	 *
	 * @return bool
	 */
	function hello_elementor_check_hide_title( $val ) {
		if ( defined( 'ELEMENTOR_VERSION' ) ) {
			$current_doc = Elementor\Plugin::instance()->documents->get( get_the_ID() );
			if ( $current_doc && 'yes' === $current_doc->get_settings( 'hide_title' ) ) {
				$val = false;
			}
		}
		return $val;
	}
}
add_filter( 'hello_elementor_page_title', 'hello_elementor_check_hide_title' );

/**
 * Wrapper function to deal with backwards compatibility.
 */
if ( ! function_exists( 'hello_elementor_body_open' ) ) {
	function hello_elementor_body_open() {
		if ( function_exists( 'wp_body_open' ) ) {
			wp_body_open();
		} else {
			do_action( 'wp_body_open' );
		}
	}
}

/**
 * Schulden: falls es nicht gefundent ist, nichts passiert.
 */
include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-user.php';

///////////////// En functions.php
    
    //Dashicons en el frontend con html como <span class="dashicons dashicons-admin-users"></span>, va en functions.php
    add_action( 'wp_enqueue_scripts', function() {
        wp_enqueue_style( 'dashicons' );
    } );

    //Controlar WP Admin Menu
{
    // keys and examples: https://whiteleydesigns.com/editing-wordpress-admin-menus/#:~:text=In%20order%20to%20rename%20menus,above%20and%20alter%20the%20name.
    // van en apariencia>editor de temas>functions.php hasta abajo... 

    //Puedes usar esta función para hacer echo de los nombres del menu (estan en array en la posición [2] siempre) y usar remove_menu_page()
    //add_action( 'admin_init', 'wpse_136058_debug_admin_menu' );
    function wpse_136058_debug_admin_menu() {
        // o  global $submenu o  global $menu
        echo '<pre>' . print_r( $GLOBALS[ 'menu' ], TRUE) . '</pre>';
    }
	//y este para el submenu
	//add_action( 'admin_init', 'wpse_136058_debug_admin_submenu' );
	function wpse_136058_debug_admin_submenu() {
        echo '<pre>' . print_r( $GLOBALS[ 'submenu' ], TRUE) . '</pre>';
	}

    /* Ocultar ciertos elementos del menu segun el usuario*/
    /* Ocultar EXCEPTO elementos del menu segun el usuario*/
    add_action( 'admin_init', 'my_remove_menu_pages_except' );
    function my_remove_menu_pages_except($query) {
         $current_user = wp_get_current_user();
        if($current_user->ID == 1) //Aqui ponemos el numero de ID del usuario en el que SI se mostrara el menu
        {//el 1 por lo general es el admin con el que creaste el WP.
            //en users al hacer hover en "edit" la url muestra el user id
            //Puedes hacer menus para cada usuario...asi
            remove_menu_page( 'index.php' );
            remove_menu_page( 'edit-comments.php' );//los ids son los links con .php , si dice page="lo que esta aqui"
            remove_menu_page( 'wc-admin&path=/analytics/overview' );//oculta Analisis Woo
            return $query;
        }else{//quitar para todos los demas
            //quitar todo excepto...
            $whitelist=array('edit.php?post_type=factura','ai1wm_export','users.php','fullstripe-settings');         
            $i=0; foreach( $GLOBALS[ 'menu' ] as $key=>$feach){
                //echo 'remove_menu_page("'.$feach['2'].'");';;echo "<br>";
                //echo !in_array($feach['2'],$whitelist);echo "<br>";
                if(!in_array($feach['2'],$whitelist) ){
                    remove_menu_page( $feach['2'] );
                }
            $i++;}
            
            //quitar del submenu... debes poder hacer algo como $GLOBALS[ 'submenu' ]['woocommerce-marketing']
            //donde woocommerce-marketing es $feach['2'] o 4 de globals...
            remove_submenu_page( 'ai1wm_export', 'ai1wm_import' );
            remove_submenu_page( 'ai1wm_export', 'ai1wmue_settings' );
            //remove_submenu_page( 'ai1wm_export', 'ai1wm_backups' );
            //remove_submenu_page( 'woocommerce-marketing', 'admin.php?page=wc-admin&path=/marketing' );
            //remove_submenu_page( 'woocommerce', 'coupons-moved' );
            
			//No puedes poner CSS aquí porque truenas cosas como la galeria          
        }
    }
	/*CSS para otros admins*/
    add_action( 'admin_head', 'css_admin_otros' );  
    function css_admin_otros() { 
		$current_user = wp_get_current_user();
        if($current_user->ID != 1) {//CSS especial para los demas o podría ser un jq remove aunque hace cosas raras ahi...
            ?> <style>
			/*Que no pueda cambiar el tema asignado ni otras cosas*/
			.user-admin-color-wrap, .show-admin-bar.user-admin-bar-front-wrap, 
			.user-rich-editing-wrap, .user-syntax-highlighting-wrap, .user-comment-shortcuts-wrap,
			.user-pass1-wrap, .user-profile-picture, #application-passwords-section, .user-url-wrap,
			.alignleft.actions #new_role, .alignleft.actions #changeit,
			.alignleft.actions #new_role2, .alignleft.actions #changeit2,
			#ac-s{
				display: none !important;
			}</style> <?php }
	}
    
    //Cambia el orden por defecto del menú admin de solo X elementos
    add_filter('custom_menu_order', function() { return true; });
    add_filter('menu_order', 'alexcd2000_custom_some_menu_order');
    /**
     * Filters WordPress' default menu order
     */
    function alexcd2000_custom_some_menu_order( $menu_order ) {
      // define your new desired menu positions here
      // for example, move 'upload.php' to position #9 and built-in pages to position #1
      $new_positions = array(
        'edit.php' => 1,
        'upload.php' => 2,
        'edit.php?post_type=page' => 3,
        'wpda' => 11,
        'njt-fs-filemanager' => 12,
        'cjtoolbox' => 13,
        'ai1wm_export' => 14,
        'edit.php?post_type=elementor_library' => 10, //templates y elementor siempre van pegados...
        'elementor' => 100,//asi aseguras quede hasta abajo
        'users.php' => 104,
        'plugins.php' => 105,
        'themes.php' => 106,
        'tools.php' => 107,
        'options-general.php' => 108
      );
      // helper function to move an element inside an array
      function move_element(&$array, $a, $b) {
        $out = array_splice($array, $a, 1);
        array_splice($array, $b, 0, $out);
      }
      // traverse through the new positions and move 
      // the items if found in the original menu_positions
      foreach( $new_positions as $value => $new_index ) {
        if( $current_index = array_search( $value, $menu_order ) ) {
          move_element($menu_order, $current_index, $new_index);
        }
      }
      return $menu_order;
    };

    //renombrar elementos del menu y submenu
    function rename_menu_items() {
        global $menu; // Global to get menu array
        foreach ( $menu as $key => $value ) { //necesitas el key por eso loop
            if ( $value[2] === 'ai1wm_export' ) {
                $menu[$key][0] = 'Backups';
            }
            if ( $value[2] === 'wpaie-main' ) {
                $menu[$key][0] = 'CSV import';
            }
            if ( $value[2] === 'cjtoolbox' ) {
                $menu[$key][0] = 'Code';
            }
            if ( $value[2] === 'wpda' ) {
                $menu[$key][0] = 'Database';
            }
            if ( $value[2] === 'njt-fs-filemanager' ) {
                $menu[$key][0] = 'FTP';
            }
        }
        global $submenu; // Global to get submenu array
        //echo "<hr><pre>";var_dump($submenu);echo "</pre><hr>";
        $submenu["wpaie-main"][0][0] = 'Import'; // Change name of first submenu
        $submenu["wpaie-main"][1][0] = 'Export (better use normal view)'; // Change name of second
    }
    add_action( 'admin_menu', 'rename_menu_items', 999 );
    
    //remove bar from top on backend
    function remove_admin_bar() { ?>
		<style type="text/css">
			#wp-admin-bar-new-content,#wp-admin-bar-wp-logo,#wp-admin-bar-site-name,#wp-admin-bar-comments, #wp-admin-bar-wpseo-menu,
			#wp-admin-bar-epc_purge_menu, .user-syntax-highlighting-wrap, 
			.user-rich-editing-wrap, .user-admin-bar-front-wrap, .user-comment-shortcuts-wrap{
				display: none !important;
			}
		</style>
		<?php $current_user = wp_get_current_user();
		if($current_user->ID != 1) //Aqui ponemos el numero de ID del usuario en el que SI se mostrara el menu
		{ ?>
		<style type="text/css">
			/*los links del post*/ body.post-type-page .edit_with_elementor, body.post-type-page .inline.hide-if-no-js, 
			body.post-type-page .trash, body.post-type-page .edit_as_new_draft, body.post-type-page .clone, 
			/*el editor dentro de la pagina*/ #elementor-editor, #elementor-switch-mode,
			/*Que también oculte en la página de paginas otras cosas*/ 
			body.post-type-page .subsubsub, body.post-type-page .tablenav, body.post-type-page .search-box, 
			/*no añadir nueva*/ body.post-type-page a.page-title-action, #menu-pages a[href="post-new.php?post_type=page"] {
				display: none !important;
			}
		</style>
		<?php }						
	}
    add_action( 'admin_head', 'remove_admin_bar' );    
}

	//agregar CSS al backend para ocultar los plugins de no actualizar... o hacer sticky algo
	add_action( 'admin_head', function () { ?> 
	<style>	
		tr[data-slug=disable-updates], tr[data-slug=disable-admin-notices-individually], .plugin-update-tr.active{
			display:none !important;/**/
		} 
        
        table.wp-list-table thead{ /*Sticky el header de la tabla*/
            position: sticky;
            top: 32px;
            background-color: #f1f1f1;
            z-index: 999;
        }
	</style> 
	<?php } );

	//permitir logout sin confirmar (are you sure you wanna log out)	
	add_action('check_admin_referer', 'logout_without_confirm', 10, 2);
	function logout_without_confirm($action, $result){
			if ($action == "log-out" && !isset($_GET['_wpnonce'])) {
				$location='/';//si tienes el plugin creo que le hace caso a eso...
				$redirect_to = isset($_REQUEST['redirect_to']) ? $_REQUEST['redirect_to'] : $location;
				$location = str_replace('&amp;', '&', wp_logout_url($redirect_to));
				header("Location: $location");
			die;}
		}

	//descativar schedule posts... no me consta...
	function onetarek_prevent_future_type( $post_data ) {
			 if ( $post_data['post_status'] == 'future' ){
				  $post_data['post_status'] = 'publish';
			 }
			 return $post_data;
		}
	add_filter('wp_insert_post_data', 'onetarek_prevent_future_type');
	remove_action('future_post', '_future_post_hook');

	//desactivar la toolbar para todos en el front
	add_filter('show_admin_bar', '__return_false');//creo es lo mismo que abajo pero mejor sobre...